@@ -1,3 +1,4 @@
+import logging
 from threading import RLock
 from plumber import plumber
 from node.parts import (
@@ -180,11 +181,22 @@ class Widget(object):
         OdictStorage,
     )
     
-    def __init__(self, extractors, edit_renderers, display_renderers, 
-                 preprocessors, uniquename=None, value_or_getter=UNSET, 
-                 properties=dict(), defaults=dict(), mode='edit'
-                 ):
+    def __init__(self,
+                 blueprints,
+                 extractors,
+                 edit_renderers,
+                 display_renderers, 
+                 preprocessors,
+                 uniquename=None,
+                 value_or_getter=UNSET, 
+                 properties=dict(),
+                 custom=dict(),
+                 defaults=dict(),
+                 mode='edit'):
         """Initialize the widget. 
+        
+        ``blueprints``
+            The blueprint names used to create this widget as list of strings.
             
         ``extractors``
             list of tuples with chain part name and callables extracting the 
@@ -221,11 +233,14 @@ class Widget(object):
             
         ``value_or_getter``
             either a callable or the value itself. If callable, its called 
-            before passing to given ``renderer`` 
+            before passing to given ``renderer`` .
                         
         ``properties``
             arbitrary dict-like passed through for use in renderer and 
             extractor, static data must never be modifed!
+        
+        ``custom``
+            Definitions for custom factory chain parts.
             
         ``defaults``
             a dict with defaults value for the widgets attributes.
@@ -238,6 +253,7 @@ class Widget(object):
         self.__name__ = uniquename
         self.__parent__ = None
         self.attributes_factory = WidgetAttributes
+        self.blueprints = blueprints
         self.getter = value_or_getter
         self.mode = mode
         self.extractors = extractors
@@ -248,9 +264,10 @@ class Widget(object):
         self._lock = RLock()
         self.current_prefix = None
         # keep properties for use in dottedpath to avoid recursion errors
-        self._properties = properties
+        self.properties = properties
         for key in properties:
             self.attrs[key] = properties[key]
+        self.custom = custom
             
     def lock(self):
         self._lock.acquire()
@@ -347,7 +364,7 @@ class Widget(object):
         path = list()
         node = self
         while node is not None:
-            if not node._properties.get('structural'):
+            if not node.properties.get('structural'):
                 path.append(node.__name__)
             node = node.__parent__
         path.reverse()
@@ -386,26 +403,49 @@ class Factory(object):
     def __init__(self):
         self._blueprints = dict()
         self._global_preprocessors = list()
+        self._plans = dict()
         self.defaults = dict()
         self.doc = {
             'props': dict(),
             'blueprint': dict(),
         }
+    
+    def _name_check(self, name):
+        for chara in '*:#':
+            if chara in name:
+                raise ValueError, '"%s" as char not allowed as name.' % chara
         
     def register(self, name, extractors=[], edit_renderers=[], 
                  preprocessors=[], builders=[], display_renderers=[]):
         """Registers a blueprint in the factory.
         """
-        if name.startswith('*'):
-            raise ValueError, 'Asterisk * as first char not allowed as name.'
-        if ':' in name:
-            raise ValueError, 'Colon : as char not allowed in name.'
+        self._name_check(name)
         self._blueprints[name] = (extractors, edit_renderers, 
                                  preprocessors, builders, display_renderers)
         
     def register_global_preprocessors(self, preprocessors):
         self._global_preprocessors += preprocessors
         
+    def register_plan(self, name, blueprints):
+        self._name_check(name)
+        self._plans[name] = blueprints
+                
+    def _expand_blueprints(self, blueprints):
+        result = list()
+        if isinstance(blueprints, basestring):
+            blueprints = blueprints.split(':')
+        for blueprint in blueprints:            
+            if blueprint.startswith('#'):
+                plan_name = blueprint[1:]
+                if plan_name not in self._plans:
+                    msg = "Plan with name '%s' is not registered in factory" % \
+                          plan_name
+                    raise ValueError(msg)
+                result += self._expand_blueprints(self._plans[plan_name])
+            else:
+                result.append(blueprint)
+        return result
+        
     def __call__(self, blueprints, 
                  name=None, 
                  value=UNSET, 
@@ -453,8 +493,7 @@ class Factory(object):
         disp_renderers = list()
         preprocessors = list()
         builders = list()
-        if isinstance(blueprints, basestring):
-            blueprints = blueprints.split(':')
+        blueprints = self._expand_blueprints(blueprints)
         for blueprint in blueprints:
             if blueprint.startswith('*'):
                 part_name = blueprint[1:]
@@ -473,13 +512,15 @@ class Factory(object):
             preprocessors = preprocessors + [(part_name, _) for _ in pre]
             builders      = builders + [(part_name, _) for _ in bui]            
         global_pre  = [('__GLOBAL__', _) for _ in self._global_preprocessors]
-        widget = Widget(extractors, 
+        widget = Widget(blueprints,
+                        extractors, 
                         edit_renderers, 
                         disp_renderers, 
                         global_pre + preprocessors,
                         uniquename=name, 
                         value_or_getter=value, 
                         properties=props,
+                        custom=custom,
                         defaults=self.defaults,
                         mode=mode)
         for part_name, builder_func in builders:
