@@ -171,7 +171,7 @@ def generic_display_renderer(widget, data):
         content = widget.attrs['template'](widget, data)
     else:
         value = fetch_value(widget, data)
-        if value is None:
+        if not value:
             value = u''
         content = widget.attrs['template'] % value
     attrs = {
@@ -892,16 +892,41 @@ disable, i.e. ``['foo', 'baz']``. Defaults to False.
 ###############################################################################
 
 def file_extractor(widget, data):
+    """Return a dict with following keys:
+    
+    mimetype
+        Mimetype of file.
+    headers
+        rfc822.Message instance.
+    original
+        Original file handle from underlying framework.
+    file
+        File descriptor containing the data.
+    filename
+        File name.
+    action
+        widget flags 'new', 'keep', 'replace', 'delete'
+    """
     name = widget.dottedpath
     if name not in data.request:
         return UNSET
-    if '%s-action' % name in data.request:
-        option = data.request.get('%s-action' % name, 'keep')
-        if option == 'keep':
-            return data.value
-        elif option == 'delete':
-            return UNSET
-    return data.request[name]
+    if not '%s-action' % name in data.request:
+        value = data.request[name]
+        if value:
+            value['action'] = 'new'
+        return value
+    value = data.value
+    action = value['action'] = data.request.get('%s-action' % name, 'keep')
+    if action == 'delete':
+        value['file'] = UNSET
+    elif action == 'replace':
+        new_val = data.request[name]
+        if new_val:
+            value = new_val
+            value['action'] = 'replace'
+        else:
+            value['action'] = 'keep'
+    return value
 
 
 @managedprops('accept', 'placeholder', 'autofocus', 
@@ -913,7 +938,6 @@ def input_file_edit_renderer(widget, data):
         'id': cssid(widget, 'input'),
         'class_': cssclasses(widget, data),            
         'type': 'file',
-        'value':  '',
         'placeholder': widget.attrs.get('placeholder') or None,
         'autofocus': widget.attrs.get('autofocus') and 'autofocus' or None,
         'required': widget.attrs.get('required') and 'required' or None,        
@@ -925,7 +949,7 @@ def input_file_edit_renderer(widget, data):
 
 def input_file_display_renderer(widget, data):
     tag = data.tag
-    # XXX TODO
+    # XXX: Display file name, size, mimetype
     return tag('div', 'DISPLAY FILE TODO')
 
 
@@ -965,11 +989,9 @@ factory.doc['blueprint']['file'] = """\
 A basic file upload blueprint.
 """
 
-factory.defaults['file.multivalued'] = False
-
 factory.defaults['file.accept'] = None
 factory.doc['props']['file.accept'] = """\
-Content type sto accept.
+Accepted mimetype.
 """
 
 factory.defaults['file.vocabulary'] = [
@@ -1234,6 +1256,8 @@ def label_renderer(widget, data):
             label_attrs['for_'] = cssid(for_widget, 'input')
         else:
             label_attrs['for_'] = cssid(widget, 'input')
+        if widget.attrs['title']:
+            label_attrs['title'] = widget.attrs['title']
     help = u''
     if widget.attrs['help']:
         help_attrs = {'class_': widget.attrs['help_class']}
@@ -1280,6 +1304,11 @@ the label.
 
 factory.defaults['label.help_class'] = 'help'
 
+factory.defaults['label.title'] = None
+factory.doc['props']['label.title'] = """\
+Optional help text to be rendered in the title attribute of the label.
+"""
+
 
 ###############################################################################
 # field
@@ -1341,4 +1370,4 @@ Renders a div with an errormessage around the prior rendered output.
 
 factory.defaults['error.error_class'] = 'error'
 
-factory.defaults['error.message_class'] = 'errormessage'
\ No newline at end of file
+factory.defaults['error.message_class'] = 'errormessage'
